@@ -4,16 +4,11 @@
 #![allow(unused_features)]
 #![deny(rust_2018_idioms)]
 #![feature(
-    asm,
-    const_fn,
-    const_fn_union,
-    const_fn_transmute,
     custom_inner_attributes,
     link_llvm_intrinsics,
     platform_intrinsics,
     repr_simd,
     simd_ffi,
-    llvm_asm,
     proc_macro_hygiene,
     stmt_expr_attributes,
     core_intrinsics,
@@ -35,13 +30,12 @@
     adx_target_feature,
     rtm_target_feature,
     f16c_target_feature,
-    external_doc,
     allow_internal_unstable,
     decl_macro,
-    extended_key_value_attributes
+    bench_black_box,
+    asm_const
 )]
 #![cfg_attr(test, feature(test, abi_vectorcall))]
-#![cfg_attr(all(test, target_arch = "wasm32"), feature(wasm_simd))]
 #![deny(clippy::missing_inline_in_public_items)]
 #![allow(
     clippy::inline_always,
@@ -73,7 +67,10 @@ extern crate std_detect;
 #[path = "mod.rs"]
 mod core_arch;
 
-pub use self::core_arch::arch;
+pub mod arch {
+    pub use crate::core_arch::arch::*;
+    pub use core::arch::asm;
+}
 
 #[allow(unused_imports)]
-use core::{ffi, hint, intrinsics, marker, mem, ops, ptr, sync};
+use core::{convert, ffi, hint, intrinsics, marker, mem, ops, ptr, sync};
